
# q-expansion of newform 126.5.n.b, downloaded from the LMFDB on 28 January 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 126
weight = 5
poly_data = [4, 0, 2, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 2, 0, 0], 1], [[0, 0, 1, 0], 2], [[0, 0, 0, 1], 1]]

hecke_ring_character_values = [[29, [1, 0, 0, 0]], [73, [1, 0, 1, 0]]]
aps_data = [[0, 1, 0, 0], [0, 0, 0, 0], [11, 1, -11, 2], [0, 7, 35, -7], [0, 6, -81, 6], [36, 68, 72, 34], [68, 34, 34, -34], [-74, 34, 74, 68], [-156, 306, -156, 0], [-681, 0, 0, 21], [-1262, 87, -631, -87], [698, 48, 698, 0], [-518, -784, -1036, -392], [-158, 0, 0, -1140], [1316, 340, -1316, 680], [0, 591, -519, 591], [322, 656, 161, -656], [848, -164, -848, -328], [0, -486, -7300, -486], [2424, 0, 0, 732], [7528, -500, 3764, 500], [1987, -2991, 1987, 0], [-2741, -1420, -5482, -710], [5526, -1218, -5526, -2436], [-1013, 80, -2026, 40], [-15624, -450, -7812, 450], [-4704, -1670, 4704, -3340], [-525, -1740, -525, 0], [0, 5550, 1000, 5550], [-5790, 0, 0, 1296], [3059, 0, 0, 1923], [-2925, -1920, 2925, -3840], [0, 7710, 8952, 7710], [-6402, -6572, -12804, -3286], [9786, -5520, 9786, 0], [0, 7689, 3439, 7689], [-6552, 6036, -3276, -6036], [21220, 7554, 21220, 0], [-16648, 1012, -33296, 506], [3492, 6954, -3492, 13908], [0, -13356, 12378, -13356], [5738, -22060, 11476, -11030], [-53664, -3852, -53664, 0], [0, 8034, 13951, 8034], [52986, 0, 0, -7128], [-16088, 2558, -8044, -2558], [62644, 0, 0, 1560], [-1383, -23946, -2766, -11973], [-49734, 3690, -24867, -3690], [4294, 5382, -4294, 10764], [7686, -354, 7686, 0], [56088, 0, 0, -14274], [28418, 6142, 14209, -6142], [-24187, -27044, -48374, -13522], [-13710, 2478, 13710, 4956], [0, 32898, -27618, 32898], [55690, 4283, 27845, -4283], [-26123, -10573, 26123, -21146], [0, 8724, 54428, 8724], [43956, 0, 0, 27690], [18020, 15576, 9010, -15576], [15235, -1294, 30470, -647], [-11888, 35508, -23776, 17754], [30212, 14194, 15106, -14194], [-17275, -15886, 17275, -31772], [-95061, 21543, -95061, 0], [-86752, -16452, -86752, 0], [20371, 0, 0, 24786], [0, 2352, -25422, 2352], [-32392, 24772, -64784, 12386], [-107128, 6508, -53564, -6508], [165780, 23454, 165780, 0], [-32218, -23845, -16109, 23845], [120118, 10170, 120118, 0], [73498, 0, 0, -23748], [-4586, 4364, 4586, 8728], [0, -7800, -110358, -7800], [72404, 18612, -72404, 37224], [-92124, -30744, -92124, 0], [51198, 1108, 25599, -1108], [-88732, 44800, -177464, 22400], [-117316, 0, 0, 35688], [0, -30702, 163578, -30702], [-88972, -25560, -177944, -12780], [9417, -1521, -9417, -3042], [237537, -22158, 237537, 0], [39144, 0, 0, -55158], [-66109, 2700, -66109, 0], [114160, 44708, 228320, 22354], [140054, 0, 0, 55800], [35364, 25764, -35364, 51528], [-52220, -33826, -26110, 33826], [0, 6807, -248141, 6807], [-159357, 0, 0, -6720], [40790, 107670, 40790, 0], [-155606, 85952, -311212, 42976], [10513, 8585, -10513, 17170], [-443996, -19840, -221998, 19840], [-213000, 18878, 213000, 37756], [283930, 79290, 283930, 0], [264878, 0, 0, 79446], [0, 103767, -215037, 103767], [167898, 54138, 83949, -54138], [-123234, -9042, -123234, 0], [0, 4356, 59972, 4356], [-397674, -43466, -198837, 43466], [-49833, -90552, -99666, -45276], [12604, -42874, -12604, -85748], [0, 45198, 206244, 45198], [-259799, -79644, -519598, -39822], [203513, 43517, -203513, 87034], [0, 1218, -410776, 1218], [-149700, 0, 0, 44832], [222312, 6404, 111156, -6404], [394283, 0, 0, 33501], [0, -81060, -348612, -81060], [-54532, 93804, -109064, 46902], [140112, -69288, 70056, 69288], [-207435, 124341, -207435, 0], [171882, 0, 0, 4416], [249524, 27938, 124762, -27938], [10981, 0, 0, -174612], [166287, 18843, -166287, 37686], [0, -19026, 288255, -19026], [-4690, 16572, 4690, 33144], [83037, 0, 0, 72567], [113246, -69744, 113246, 0], [-222050, 76448, 222050, 152896], [-211003, 111486, -422006, 55743], [-237502, -84654, 237502, -169308], [0, -140922, 371402, -140922], [-81012, 0, 0, 244938], [-284233, 30543, -284233, 0], [-235964, 0, 0, 189060], [-187928, -97498, 187928, -194996], [408313, 103972, 816626, 51986], [-518152, 122434, -259076, -122434], [-1688, -40300, -844, 40300], [266823, 215802, 533646, 107901], [0, 30864, 4992, 30864], [-42732, -221724, -85464, -110862], [150843, 55233, 150843, 0], [0, 321144, -61474, 321144], [-566403, 0, 0, -96288], [86996, 195764, 43498, -195764], [236426, -292616, 472852, -146308], [252944, -149980, 505888, -74990], [375632, -133736, 187816, 133736], [-145418, -37132, 145418, -74264], [-537690, -82920, -537690, 0], [-298760, 19884, -298760, 0], [-94260, -197292, -188520, -98646], [-773966, 0, 0, -110340], [-308596, -153038, 308596, -306076], [0, 185586, 573352, 185586], [297720, 0, 0, -362532], [-302122, -75804, -302122, 0], [-618142, 34444, 618142, 68888], [25767, -488768, 51534, -244384], [-308162, 32531, -154081, -32531], [-192642, -77772, -192642, 0], [-1365726, 0, 0, 22908], [59273, 0, 0, 142767], [401251, -154762, -401251, -309524], [0, -18864, -1492350, -18864], [-308804, 27182, -154402, -27182], [0, -174963, -296321, -174963], [916568, -24702, 458284, 24702]]
